// 
// Copyright (C) 2005 and onwards Google Inc.
// Author: Brian Ryner
//
// This Javascript XPCOM component handles registering the appropriate
// platform-specific component location for the Google Toolbar.
//
// The component libraries are separated into directories
// components.[win|mac|lin] for Windows, Mac, and Linux.  We figure out
// the current platform and register the appropriate location with XPCOM.
// This is all necessary to avoid having Windows try to load a non-Windows
// library, which causes an error dialog to appear.

var nsIComponentRegistrar = Components.interfaces.nsIComponentRegistrar;

var UNINSTALLER_CID = Components.ID("{abf0defb-ad5c-4673-a8fe-edd64c22a441}");
var UNINSTALLER_NAME = "Google Toolbar Uninstall Handler";
var UNINSTALLER_CONTRACTID = "@google.com/uninstaller;1";


var gModule = {
  getComponentDir: function(location) {
    // HACK ALERT
    // Because this method may be called from registerSelf, which runs prior to
    // the "xpcom-startup" event being dispatched, GDS may not have initialized
    // itself yet.  The act of loading the HTTP protocol handler may result in
    // large portions of Mozilla being initialized, some of which may end up
    // tickling GDS.  GDS gets very angry if it has not been initialized, so we
    // attempt to avoid this problem here.
    try {
      Components.classes["@google/googledesktopmozilla;1"].getService();
    } catch (e) {
      // Maybe GDS is not installed.
    }

    // Find the current platform.  We can't use "navigator" because it is
    // not defined in this context.  So we use the HttpPrototocolHandler (ugh).
    var platform =
        Components.classes["@mozilla.org/network/protocol;1?name=http"].
            getService(Components.interfaces.nsIHttpProtocolHandler).oscpu;

    var compdir = null;
    if (platform.indexOf("Win") != -1) {
      compdir = "win";
    } else if (platform.indexOf("Mac") != -1) {
      compdir = "mac";
    } else if (platform.indexOf("Linux") != -1) {
      compdir = "lin";
    }

    if (!compdir)
      return null;

    // Find the directory above "components", then locate the platform
    // components directory
    var extensiondir = location.parent.parent;
    extensiondir.append("components." + compdir);

    return extensiondir;
  },

  getClassObject: function(compMgr, clazz, iid) {
    if (!clazz.equals(UNINSTALLER_CID)) {
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }

    if (!iid.equals(Components.interfaces.nsIFactory)) {
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }

    return this.uninstallerFactory;
  },

  registerSelf: function(compMgr, location, loaderStr, type) {
    var manager = compMgr.QueryInterface(nsIComponentRegistrar);

    // Register the new library
    var compdir = this.getComponentDir(location);
    if (compdir) {
      manager.autoRegister(compdir);
    }

    manager.registerFactoryLocation(
        UNINSTALLER_CID,
        UNINSTALLER_NAME,
        UNINSTALLER_CONTRACTID,
        location, loaderStr, type);
  },

  unregisterSelf: function(compMgr, location, loaderStr) {
    // Unregister the new library
    var compdir = this.getComponentDir(location);
    if (compdir)
      compMgr.QueryInterface(nsIComponentRegistrar).autoUnregister(compdir);
  },

  canUnload: function(compMgr) {
    return true;
  },

  QueryInterface: function(iid) {
    if (iid.equals(Components.interfaces.nsIModule) ||
        iid.equals(Components.interfaces.nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  /**
   * nsIFactory
   */
  uninstallerFactory: {
    QueryInterface: function(iid) {
      if (iid.equals(Components.interfaces.nsIFactory) ||
          iid.equals(Components.interfaces.nsISupports))
      return this;
      
      throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    createInstance: function(outer, iid) {
      if (outer != null) {
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      }
      
      if (!this.uninstaller) {
        this.uninstaller = {
          _prefs: [
              "google.toolbar.AUTOFILL_Preferences",
              "google.toolbar.opted_into_advanced_features_1",
              "google.toolbar.search_box_history",
              "google.toolbar.first_search",
              "google.toolbar.install_ping"
              ],
              
          observe: function(subject, topic, data) {
            if (topic == "xpcom-shutdown") {
              // components we need for accessing the extension data
              var service = Components.classes["@mozilla.org/rdf/rdf-service;1"]
              .getService(Components.interfaces.nsIRDFService);
              var container = Components.classes["@mozilla.org/rdf/container;1"]
              .getService(Components.interfaces.nsIRDFContainer);
              var datasource= Components.classes["@mozilla.org/extensions/manager;1"]
              .getService(Components.interfaces.nsIExtensionManager).datasource;
              var root = service.GetResource("urn:mozilla:extension:root");

              // attributes we need to access
              // i'd rather use id, but it doesn't seem to be there!
              var nameResource = service
              .GetResource("http://www.mozilla.org/2004/em-rdf#name");
              var uninstallResource = service
              .GetResource("http://www.mozilla.org/2004/em-rdf#toBeUninstalled");

              container.Init(datasource, root);
              var elements = container.GetElements();
              while (elements.hasMoreElements()) {
                var element = elements.getNext()
                              .QueryInterface(Components.interfaces.nsIRDFResource);

                // TODO(nowitz) the name might even change in other locales
                //  find a better way to do this!
                var target = datasource.GetTarget(element, nameResource, true);
                if (!target
                    || target.QueryInterface(Components.interfaces.nsIRDFLiteral)
                    .Value != "Google Toolbar for Firefox") {
                  // not our extension
                  continue;
                }

                target = datasource.GetTarget(element, uninstallResource, true);
                if (!target
                    || target.QueryInterface(Components.interfaces.nsIRDFLiteral)
                    .Value != "true") {
                  // not pending uninstall
                  continue;
                }

                // if we got this far in the while loop,
                //  our extension is pending uninstall
                var psvc = Components.classes["@mozilla.org/preferences;1"]
                           .getService(Components.interfaces.nsIPrefBranch);
                for (var i = 0; i < this._prefs.length; i++) {
                  try {
                    psvc.clearUserPref(this._prefs[i]);
                  } catch (e) {
                    // if there is no preference, that's ok, too
                  }
                }
                
                psvc = psvc.QueryInterface(Components.interfaces.nsIPrefService);
                try {
                  psvc.savePrefFile(null);
                } catch (e) {

                }
              }
            }
          },

          QueryInterface: function(iid) {
            if (iid.equals(Components.interfaces.nsISupports) ||
                iid.equals(Components.interfaces.nsIObserver)) {
              return this;
            }

            throw Components.results.NS_ERROR_NO_INTERFACE;
          }
        };

        var obs = Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
        obs.addObserver(this.uninstaller, "xpcom-shutdown", false);
      }

      return this.uninstaller.QueryInterface(iid);
    },

    uninstaller: null,
  },
};

function NSGetModule(compMgr, location) {
  return gModule;
}
